from ultipa.types import ULTIPA
from ultipa.types.types import OrderType, UltipaEquation
from ultipa.utils import common as COMMON
from ultipa.utils.ufilter import ufilter as FILTER
from ultipa.utils.ufilter.new_ufilter import *


class OrderBy:
	def __init__(self, schemaName: str, propertyName: str, orderType: OrderType):
		self.schemaName = '@' + schemaName
		self.propertyName = propertyName
		self.orderType = orderType.value

	@property
	def toString(self):
		return "%s.%s %s" % (self.schemaName, self.propertyName, self.orderType)


class GroupBy:
	def __init__(self, schemaName: str, propertyName: str):
		self.schemaName = '@' + schemaName
		self.propertyName = propertyName

	@property
	def toString(self):
		return "%s.%s" % (self.schemaName, self.propertyName)


class CommonSchema:
	def __init__(self, schema: str, property: str):
		self.schemaName = '@' + schema
		self.propertyName = property

	@property
	def toString(self):
		return "%s.%s" % (self.schemaName, self.propertyName)


class UltipaPath:
	def __init__(self, nodeSchema: List[CommonSchema], edgeSchema: List[CommonSchema]):
		self.nodeSchema = nodeSchema
		self.edgeSchema = edgeSchema

	@property
	def toString(self):
		if self.nodeSchema == '*':
			nodeSchema = '*'
		else:
			nodeSchema = ','.join([i.toString for i in self.nodeSchema])

		if self.edgeSchema == '*':
			edgeSchema = '*'
		else:
			edgeSchema = ','.join([i.toString for i in self.edgeSchema])

		return "{%s}{%s}" % (nodeSchema, edgeSchema)


class UltipaReturn:
	def __init__(self, schemaName: str, propertyName: str, equation: UltipaEquation = None, alias: List[str] = None,
				 path: UltipaPath = None):
		self.schemaName = '@' + schemaName
		self.propertyName = propertyName
		self.equation = equation
		self.alias = alias
		self.path = path

	@property
	def toString(self):
		if self.path:
			return "path%s" % (self.path.toString)
		if self.alias and self.propertyName and not self.schemaName and not self.equation:
			return "%s{%s}" % (self.alias, ','.join(self.propertyName))

		if self.equation and self.alias:
			return "%s(%s.%s) as %s" % (self.equation.value, self.schemaName, self.propertyName, self.alias)
		return "{%s.%s}" % (self.schemaName, self.propertyName)


class Return:
	def __init__(self, alias: str, propertys: List[str] = None, allProperties: bool = False, limit: int = COMMON.LIMIT):
		if propertys is None:
			propertys = []
		self.aliasName = alias
		self.propertys = propertys
		self.all = allProperties
		self.limit = limit

	@property
	def toString(self):
		if self.all:
			return "%s{%s} limit %s" % (self.aliasName, "*", self.limit)
		if len(self.propertys) == 1:
			return "%s.%s limit %s" % (self.aliasName, self.propertys[0], self.limit)
		else:
			return "%s{%s} limit %s" % (self.aliasName, ','.join(self.propertys), self.limit)




class CreateUser:
	def __init__(self, username: str, password: str, graphPrivileges: [dict] = None,
				 systemPrivileges: List[str] = None, policies: List[str] = None):
		self.username = username
		self.password = password
		self.graph_privileges = graphPrivileges
		self.system_privileges = systemPrivileges
		self.policies = policies


class DropUser:
	def __init__(self, username: str):
		self.username = username


class AlterUser(CreateUser):
	def __init__(self, username: str, password: str = None, graph_privileges: [dict] = None,
				 system_privileges: List[str] = None, policies: List[str] = None):
		super().__init__(username, password, graph_privileges, system_privileges, policies)


class GetUser:
	def __init__(self, username: str):
		self.username = username


class GetUserSetting:
	def __init__(self, username: str, type: str):
		self.username = username
		self.type = type


class SetUserSetting:
	def __init__(self, username: str, type: str, data: str):
		self.username = username
		self.type = type
		self.data = data


class Kill:
	def __init__(self, id: str = None, all: bool = False):
		self.id = id
		self.all = all


class ShowTask:
	def __init__(self, id: int = None, name: str = None, limit: int = None, status: str = ''):
		self.id = id
		self.limit = limit
		self.name = name
		self.status = status


class ClearTask:
	def __init__(self, id: int = None, name: str = None, status: str = None, all: bool = False):
		self.id = id
		self.name = name
		self.status = status
		self.all = all


class PauseTask:
	def __init__(self, id: int = None, all: bool = None):
		self.id = id
		self.all = all


class ResumeTask:
	def __init__(self, id: int = None, all: bool = None):
		self.id = id
		self.all = all


class StopTask():
	def __init__(self, id: int = None, all: bool = False):
		self.id = id
		self.all = all


class Header:
	def __init__(self, name: str, type: ULTIPA.PropertyType):
		self.name = name
		self.type = type


class InsertNodeBulk:
	def __init__(self, schema: str, rows: List[dict], insertType: ULTIPA.InsertType, silent: bool = False,
				 batch: bool = False, n: int = 100, timeZone=None, timeZoneOffset=None):
		self.schema = schema
		self.rows = rows
		self.silent = silent
		self.insertType = insertType
		self.batch = batch
		self.n = n
		self.timeZone = timeZone
		self.timeZoneOffset = timeZoneOffset


class InsertEdgeBulk:
	def __init__(self, schema: str, rows: List[dict], insertType: ULTIPA.InsertType, silent: bool = False,
				 create_node_if_not_exist: bool = False, batch: bool = False, n: int = 100, timeZone=None,
				 timeZoneOffset=None):
		self.schema = schema
		self.rows = rows
		self.silent = silent
		self.create_node_if_not_exist = create_node_if_not_exist
		self.insertType = insertType
		self.batch = batch
		self.n = n
		self.timeZone = timeZone
		self.timeZoneOffset = timeZoneOffset


class InsertNode:
	def __init__(self, nodes: List[dict], schema: str, overwrite: bool = False, upsert: bool = False,
				 isReturnID: bool = True):
		self.nodes = nodes
		self.schemaName = '@' + schema
		self.overwrite = overwrite
		self.upsert = upsert
		self.isReturnID = isReturnID

	def setSchema(self, schema: str):
		self.schemaName = '@' + schema


class InsertEdge:
	def __init__(self, edges: List[dict], schema: str, overwrite: bool = False, upsert: bool = False,
				 isReturnID: bool = True):
		self.edges = edges
		self.schemaName = '@' + schema
		self.overwrite = overwrite
		self.upsert = upsert
		self.isReturnID = isReturnID

	def setSchema(self, schema: str):
		self.schemaName = '@' + schema


class SearchNode:
	def __init__(self, select: Return, id=None,
				 filter: UltipaFilter or list or str = None):
		if id is None:
			id = []
		self.id = id
		self.filter = filter
		self.select = select


class SearchEdge(SearchNode):
	pass


class UpdateNode:
	def __init__(self, values: dict, uuid: [int] = None, filter: UltipaFilter = None, silent: bool = False):
		if uuid is None:
			uuid = []
		self.id = uuid
		self.filter = filter
		self.values = values
		self.silent = silent


class UpdateEdge(UpdateNode):
	pass


class DeleteNode:
	def __init__(self, uuid: [int] = None, filter: UltipaFilter = None, silent: bool = False):
		if uuid is None:
			uuid = []
		self.id = uuid
		self.filter = filter
		self.silent = silent


class DeleteEdge(DeleteNode):
	pass


class Graph:
	def __init__(self, graph, description: str = None):
		self.graph = graph
		self.description = description


class AlterGraph:
	def __init__(self, oldGraphName: str, newGraphName: str, newDescription: str = None):
		self.oldGraphName = oldGraphName
		self.newGraphName = newGraphName
		self.newDescription = newDescription


class Retry:
	def __init__(self, current: int = 0, max: int = 3):
		self.current = current
		self.max = max


class RequestConfig:

	def __init__(self, graphName: str = '', timeout: int = 3600, retry: Retry = Retry(),
				 stream: bool = False, host: str = None, useMaster: bool = False, threadNum: int = None,
				 timeZone: str = None, timeZoneOffset: any = None):
		self.graphName = graphName
		self.timeoutWithSeconds = timeout
		self.retry = retry
		self.stream = stream
		self.useHost = host
		self.useMaster = useMaster
		self.threadNum = threadNum
		self.timeZone = timeZone
		self.timeZoneOffset = timeZoneOffset


class InsertConfig(RequestConfig):
	def __init__(self, insertType: ULTIPA.InsertType, graphName: str = '', timeout: int = 3600,
				 retry: Retry = Retry(), stream: bool = False, useHost: str = None, useMaster: bool = False,
				 CreateNodeIfNotExist: bool = False, timeZone=None, timeZoneOffset=None, **kwargs):
		super().__init__(graphName, timeout, retry, stream, useHost, useMaster, timeZone=timeZone,
						 timeZoneOffset=timeZoneOffset)
		self.insertType = insertType
		if kwargs.get("silent") is not None:
			self.silent = kwargs.get("silent")
		else:
			self.silent = True
		if kwargs.get("batch") is not None:
			self.batch = kwargs.get("batch")
		if kwargs.get("n") is not None:
			self.n = kwargs.get("n")
		if kwargs.get("timeoutWithSeconds") is not None:
			self.timeoutWithSeconds = kwargs.get("timeoutWithSeconds")
		self.createNodeIfNotExist = CreateNodeIfNotExist


class Common(InsertConfig):
	...


class LTE:

	def __init__(self, schemaName: CommonSchema, type: ULTIPA.DBType):
		'''LTE UFE Node and Edge property'''
		self.schemaName = schemaName
		self.type = type


class UFE(LTE):
	...


class Property:
	def __init__(self, type: ULTIPA.DBType, name: str = ''):
		self.type = type
		self.name = name


class GetProperty:
	def __init__(self, type: ULTIPA.DBType, schema: str = None):
		self.type = type
		self.schemaName = schema

	def __str__(self):
		return '@' + self.schemaName


class CreateProperty(Property):
	def __init__(self, type: ULTIPA.DBType, commonSchema: CommonSchema,
				 propertyType: ULTIPA.CreatePropertyType = ULTIPA.CreatePropertyType.PROPERTY_STRING,
				 description: str = '', subTypes: List[ULTIPA.CreatePropertyType] = None):
		super().__init__(type)
		self.propertyType = propertyType
		self.subTypes = subTypes
		self.description = description
		self.schemaName = commonSchema


class DropProperty(Property):
	def __init__(self, type: ULTIPA.DBType, commonSchema: CommonSchema):
		super().__init__(type)
		self.schemaName = commonSchema


class AlterProperty(Property):
	def __init__(self, type: ULTIPA.DBType, commonSchema: CommonSchema, newName: str, newDescription: str = ''):
		super().__init__(type)
		self.new_name = newName
		self.description = newDescription
		self.schemaName = commonSchema


class CreateSchema:
	def __init__(self, name: str, type: ULTIPA.DBType, description: str = None):
		self.schemaName = name
		self.type = type
		self.description = description


class ShowSchema:
	def __init__(self, type: ULTIPA.DBType = None, schema: str = None):
		self.schemaType = type
		self.schemaName = schema


class AlterSchema:
	def __init__(self, type: ULTIPA.DBType, schema: str, newName: str, newDescription: str = ''):
		self.type = type
		self.schemaName = schema
		self.new_schemaName = newName
		self.description = newDescription


class DropSchema:
	def __init__(self, type: ULTIPA.DBType, schema: str):
		self.type = type
		self.schemaName = schema


class Index(CommonSchema):
	def __init__(self, type: ULTIPA.DBType, schema: str, property: str):
		super().__init__(schema=schema, property=property)
		self.DBtype = type


class ShowIndex():
	def __init__(self, type: ULTIPA.DBType):
		self.DBtype = type


class ShowFulltext(ShowIndex):
	def __init__(self, type: ULTIPA.DBType):
		super().__init__(type=type)


class CreateIndex(Index):
	def __init__(self, type: ULTIPA.DBType, schema: str, property: str):
		super().__init__(type, schema, property)


class CreateFulltext(Index):
	def __init__(self, type: ULTIPA.DBType, schema: str, property: str, name: str):
		super().__init__(type, schema, property)
		self.name = name


class DropIndex(Index):
	def __init__(self, type: ULTIPA.DBType, schema: str, property: str):
		super().__init__(type, schema, property)


class DropFulltext:
	def __init__(self, type: ULTIPA.DBType, name: str = ""):
		self.fulltextName = name
		self.DBtype = type



class SearchAB:
	def __init__(self, src: int = None, dest: int = None, depth: int = COMMON.DEPTH, limit: int = COMMON.LIMIT,
				 select: List[str] = None,
				 select_node_properties: List[str] = None, select_edge_properties: List[str] = None,
				 shortest: bool = False, nodeFilter: dict = None,
				 edgeFilter: dict = None, path_ascend: str = '', path_descend: str = '',
				 direction: ULTIPA.DirectionType = None, turbo: bool = False, osrc: str = '',
				 odest: str = '', no_circle: bool = False, boost: bool = False):
		self.src = src
		self.dest = dest
		self.depth = depth
		self.shortest = shortest
		self.node_filter = nodeFilter
		self.edge_filter = edgeFilter
		self.path_ascend = path_ascend
		self.path_descend = path_descend
		self.direction = direction
		self.no_circle = no_circle


class Searchkhop:
	def __init__(self, src: int = None, depth: int = COMMON.DEPTH, limit: int = COMMON.LIMIT,
				 select: List[str] = None, select_node_properties: List[str] = None,
				 select_edge_properties: List[str] = None,
				 node_filter: dict = None, edge_filter: dict = None, direction: ULTIPA.DirectionType = None,
				 turbo: bool = False, osrc: str = ''):
		self.src = src
		self.depth = depth
		self.limit = limit
		self.select = select
		self.select_node_properties = select_node_properties
		self.select_edge_properties = select_edge_properties
		self.node_filter = node_filter
		self.edge_filter = edge_filter
		self.direction = direction
		self.turbo = turbo
		self.osrc = osrc


class Download:
	def __init__(self, fileName: str, taskId: str, savePath: str = None):
		self.fileName = fileName
		self.taskId = taskId
		self.savePath = savePath


class Policy:

	def __init__(self, name: str, graphPrivileges: dict = None, systemPrivileges: List[str] = None,
				 policies: List[str] = None):
		self.name = name
		self.graph_privileges = graphPrivileges
		self.system_privileges = systemPrivileges
		self.policies = policies


class CreatePolicy(Policy):
	pass


class AlterPolicy(Policy):
	pass


class GetPolicy:
	def __init__(self, name: str):
		self.name = name


class DropPolicy(GetPolicy):
	pass


class GrantPolicy:
	def __init__(self, username: str = '', graphPrivileges: dict = None,
				 systemPrivileges: List[str] = None, policies: List[str] = None):
		self.username = username
		self.graph_privileges = graphPrivileges
		self.system_privileges = systemPrivileges
		self.policies = policies


class RevokePolicy(GrantPolicy):
	pass


class NodeSpread:
	def __init__(self, src: int = None, depth: int = COMMON.DEPTH, limit: int = COMMON.LIMIT,
				 select: List[str] = None, selectNodeProperties: List[str] = None,
				 selectEdgeProperties: List[str] = None, nodeFilter: FILTER = None, edgeFilter: FILTER = None,
				 spread_type: str = None,
				 direction: ULTIPA.DirectionType = None, osrc: str = None):
		self.src = src
		self.depth = depth
		self.limit = limit
		self.select = select
		self.select_node_properties = selectNodeProperties
		self.select_edge_properties = selectEdgeProperties
		self.node_filter = nodeFilter
		self.edge_filter = edgeFilter
		self.spread_type = spread_type
		self.direction = direction
		self.osrc = osrc


class AutoNet:
	def __init__(self, srcs: List[int], dests: List[int] = None, depth: int = COMMON.DEPTH,
				 limit: int = COMMON.LIMIT,
				 select: List[str] = None, selectNodeProperties: List[str] = None,
				 selectEdgeProperties: List[str] = None,
				 shortest: bool = False, nodeFilter: FILTER = None, edgeFilter: FILTER = None,
				 turbo: bool = False, noCircle: bool = False, boost: bool = False):
		self.srcs = srcs
		self.dests = dests
		self.depth = depth
		self.limit = limit
		self.select = select
		self.select_node_properties = selectNodeProperties
		self.select_edge_properties = selectEdgeProperties
		self.shortest = shortest
		self.node_filter = nodeFilter
		self.edge_filter = edgeFilter
		self.turbo = turbo
		self.no_circle = noCircle
		self.boost = boost


class Export:
	def __init__(self, type: ULTIPA.DBType, limit: int, schema: str, properties: List[str] = None):
		self.type = type
		self.limit = limit
		self.properties = properties
		self.schema = schema


class TEdgeItem:
	e = 'e'
	le = 'le'
	re = 're'


class TNodeItem:
	n = 'n'


class TemplateBaseItem:
	def __init__(self, name, alias: str, filter=None):
		self.name = name
		self.alias = alias
		self.filter = filter


class TemplateEdgeItem(TemplateBaseItem):
	def __init__(self, name: TEdgeItem, alias: str = '', filter=None, nodeFilter=None, steps: List[str] = None):
		super().__init__(name=name, alias=alias, filter=filter)
		self.node_filter = nodeFilter
		self.steps = steps


class TemplateNodeItem(TemplateBaseItem):
	...


class Template:
	def __init__(self, alias: str, items: List[TemplateEdgeItem or TemplateNodeItem], limit: int, _return,
				 order_by: any = None, isKhopTemplate: bool = False, select: list = None
				 ):
		self.alias = alias
		self.items = items
		self.limit = limit
		self.order_by = order_by
		self._return = _return
		self.isKhopTemplate = isKhopTemplate
		self.select = select


class Truncate:
	def __init__(self, graph: str, truncateType: ULTIPA.TruncateType = None, allData: bool = False, schema: str = None):
		self.dbType = truncateType
		self.graphSetName = graph
		self.all = allData
		self.schema = schema


class Mount:
	def __init__(self, graph: str):
		self.graph = graph


class Unmount:
	def __init__(self, graph: str):
		self.graph = graph


class InstallAlgo:
	def __init__(self, configPath: str, soPath: str):
		self.configPath = configPath
		self.soPath = soPath


class InstallExtaAlgo(InstallAlgo):
	...


class UninstallAlgo:

	def __init__(self, algoName: str):
		self.algoName = algoName


class UninstallExtaAlgo(UninstallAlgo):
	...


class Property:
	PropertyMap = {
		"string": ULTIPA.PropertyType.PROPERTY_STRING,
		"int32": ULTIPA.PropertyType.PROPERTY_INT32,
		"int64": ULTIPA.PropertyType.PROPERTY_INT64,
		"uint32": ULTIPA.PropertyType.PROPERTY_UINT32,
		"uint64": ULTIPA.PropertyType.PROPERTY_UINT64,
		"float": ULTIPA.PropertyType.PROPERTY_FLOAT,
		"double": ULTIPA.PropertyType.PROPERTY_DOUBLE,
		"datetime": ULTIPA.PropertyType.PROPERTY_DATETIME,
		"timestamp": ULTIPA.PropertyType.PROPERTY_TIMESTAMP,
		"text": ULTIPA.PropertyType.PROPERTY_TEXT,
		"_id": ULTIPA.PropertyType.PROPERTY_ID,
		"_uuid": ULTIPA.PropertyType.PROPERTY_UUID,
		"_from": ULTIPA.PropertyType.PROPERTY_FROM,
		"_to": ULTIPA.PropertyType.PROPERTY_TO,
		"_from_uuid": ULTIPA.PropertyType.PROPERTY_FROM_UUID,
		"_to_uuid": ULTIPA.PropertyType.PROPERTY_TO_UUID,
		"_ignore": ULTIPA.PropertyType.PROPERTY_IGNORE,
		"unset": ULTIPA.PropertyType.PROPERTY_UNSET,
		"point": ULTIPA.PropertyType.PROPERTY_POINT,
		"decimal": ULTIPA.PropertyType.PROPERTY_DECIMAL,
		"list": ULTIPA.PropertyType.PROPERTY_LIST,
		"set": ULTIPA.PropertyType.PROPERTY_SET,
		"map": ULTIPA.PropertyType.PROPERTY_MAP,
		"null": ULTIPA.PropertyType.PROPERTY_NULL,
	}

	PropertyReverseMap = {
		ULTIPA.PropertyType.PROPERTY_STRING: "string",
		ULTIPA.PropertyType.PROPERTY_INT32: "int32",
		ULTIPA.PropertyType.PROPERTY_INT64: "int64",
		ULTIPA.PropertyType.PROPERTY_UINT32: "uint32",
		ULTIPA.PropertyType.PROPERTY_UINT64: "uint64",
		ULTIPA.PropertyType.PROPERTY_FLOAT: "float",
		ULTIPA.PropertyType.PROPERTY_DOUBLE: "double",
		ULTIPA.PropertyType.PROPERTY_DATETIME: "datetime",
		ULTIPA.PropertyType.PROPERTY_TIMESTAMP: "timestamp",
		ULTIPA.PropertyType.PROPERTY_TEXT: "text",
		ULTIPA.PropertyType.PROPERTY_ID: "_id",
		ULTIPA.PropertyType.PROPERTY_UUID: "_uuid",
		ULTIPA.PropertyType.PROPERTY_FROM: "_from",
		ULTIPA.PropertyType.PROPERTY_TO: "_to",
		ULTIPA.PropertyType.PROPERTY_FROM_UUID: "_from_uuid",
		ULTIPA.PropertyType.PROPERTY_TO_UUID: "_to_uuid",
		ULTIPA.PropertyType.PROPERTY_IGNORE: "_ignore",
		ULTIPA.PropertyType.PROPERTY_UNSET: "unset",
		ULTIPA.PropertyType.PROPERTY_POINT: "point",
		ULTIPA.PropertyType.PROPERTY_DECIMAL: "decimal",
		ULTIPA.PropertyType.PROPERTY_LIST: "list",
		ULTIPA.PropertyType.PROPERTY_SET: "set",
		ULTIPA.PropertyType.PROPERTY_MAP: "map",
		ULTIPA.PropertyType.PROPERTY_NULL: "null",
	}

	def __init__(self, name: str, type: ULTIPA.PropertyType = None, desc: str = None,
				 subTypes: List[ULTIPA.PropertyType] = None):
		self.name = name
		self.desc = desc
		self.type = type
		self.subTypes = subTypes

	def setSubTypesbyType(self, type: str):
		if "string" in type:
			self.subTypes = [ULTIPA.PropertyType.PROPERTY_STRING]

		if "int32" in type:
			self.subTypes = [ULTIPA.PropertyType.PROPERTY_INT32]

		if "uint32" in type:
			self.subTypes = [ULTIPA.PropertyType.PROPERTY_UINT32]

		if "int64" in type:
			self.subTypes = [ULTIPA.PropertyType.PROPERTY_INT64]

		if "uint64" in type:
			self.subTypes = [ULTIPA.PropertyType.PROPERTY_UINT64]

		if "float" in type:
			self.subTypes = [ULTIPA.PropertyType.PROPERTY_FLOAT]

		if "double" in type:
			self.subTypes = [ULTIPA.PropertyType.PROPERTY_DOUBLE]

		if "datetime" in type:
			self.subTypes = [ULTIPA.PropertyType.PROPERTY_DATETIME]

		if "timestamp" in type:
			self.subTypes = [ULTIPA.PropertyType.PROPERTY_TIMESTAMP]

		if "text" in type:
			self.subTypes = [ULTIPA.PropertyType.PROPERTY_TEXT]

	def isIdType(self) -> bool:
		idTypes = [
			ULTIPA.PropertyType.PROPERTY_ID,
			ULTIPA.PropertyType.PROPERTY_TO,
			ULTIPA.PropertyType.PROPERTY_UUID,
			ULTIPA.PropertyType.PROPERTY_FROM,
			ULTIPA.PropertyType.PROPERTY_FROM_UUID,
			ULTIPA.PropertyType.PROPERTY_TO_UUID,
		]
		return self.type in idTypes

	def isIgnore(self):
		return self.type == ULTIPA.PropertyType.PROPERTY_IGNORE

	def setTypeStr(self, value):
		self.type = self.getStringByPropertyType(value)

	def setTypeInt(self, value):
		self.type = self.getPropertyTypeByString(value)

	def getStringType(self):
		return self.getStringByPropertyType(self.type)

	def getPropertyTypeByString(self, v):
		if not self.PropertyMap.get(v):
			if "[" in v:
				self.setSubTypesbyType(v)
				return ULTIPA.PropertyType.PROPERTY_LIST

		return self.PropertyMap.get(v)

	def getStringByPropertyType(self, v):
		return self.PropertyReverseMap[v]

	@staticmethod
	def _getStringByPropertyType(v):
		return Property.PropertyReverseMap[v]

	@staticmethod
	def _getPropertyTypeByString(v):
		return Property.PropertyMap.get(v)




class Schema:

	def __init__(self, name: str, properties: List[Property], desc: str = None, type: str = None,
				 DBType: ULTIPA.DBType = None, total: int = None):
		self.name = name
		self.properties = properties
		self.desc = desc
		self.type = type
		self.DBType = DBType
		self.total = total

	def getProperty(self, name: str):
		find = list(filter(lambda x: x.get('name') == name, self.properties))
		if find:
			return find[0]
		return None


class Batch:
	Nodes: List[ULTIPA.EntityRow]
	Edges: List[ULTIPA.EntityRow]
	Schema: Schema

	def __init__(self, Schema: Schema = None, Nodes: List[ULTIPA.EntityRow] = None,
				 Edges: List[ULTIPA.EntityRow] = None):
		if Nodes is None:
			Nodes = []
		if Edges is None:
			Edges = []
		self.Nodes = Nodes
		self.Edges = Edges
		self.Schema = Schema


class InsertNodeTable:
	def __init__(self, schemas: List[Schema], nodeRows: List[ULTIPA.Node]):
		self.schemas = schemas
		self.nodeRows = nodeRows


class InsertEdgeTable:
	def __init__(self, schemas: List[Schema], edgeRows: List[ULTIPA.Edge]):
		self.schemas = schemas
		self.edgeRows = edgeRows



from ultipa.operate.base_extra import BaseExtra
from ultipa.types import ULTIPA_REQUEST, ULTIPA, ULTIPA_RESPONSE
from ultipa.utils import UQLMAKER, CommandList
import json

from ultipa.utils.convert import convertToAnyObject
from ultipa.utils.ResposeFormat import ResponseKeyFormat


class ALGO_RETURN_TYPE:
	ALGO_RETURN_REALTIME = 1
	ALGO_RETURN_WRITE_BACK = 2
	ALGO_RETURN_VISUALIZATION = 4


class TaskExtra(BaseExtra):

	def top(self,
			requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ResponseListTop:
		uqlMaker = UQLMAKER(command=CommandList.top, commonParams=requestConfig)
		res = self.UqlListSimple(uqlMaker)
		return res

	def kill(self, request: ULTIPA_REQUEST.Kill,
			 requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ResponseCommon:
		commonP = []
		if request.id:
			commonP = request.id
		if request.all:
			commonP = '*'
		uqlMaker = UQLMAKER(command=CommandList.kill, commonParams=requestConfig)
		uqlMaker.setCommandParams(commonP)
		res = self.uqlSingle(uqlMaker)
		return res

	def showTask(self, request: ULTIPA_REQUEST.ShowTask,
				 requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ResponseListTask:
		'''
		:EXP: conn.showTask(ULTIPA_REQUEST.ShowTask(limit=10))
		:param req: ULTIPA_REQUEST
		:return:
		'''
		_jsonKeys = ['taskJson']
		uqlMaker = UQLMAKER(command=CommandList.showTask, commonParams=requestConfig)
		commonP = []
		if request.name and request.status:
			commonP.append(request.name)
			commonP.append(request.status)
		if request.name and not request.status:
			commonP.append(request.name)
			commonP.append('*')
		if not request.name and request.status:
			commonP.append('*')
			commonP.append(request.status)
		if request.id:
			commonP = request.id
		uqlMaker.setCommandParams(commandP=commonP)
		res = self.UqlListSimple(uqlMaker=uqlMaker, responseKeyFormat=ResponseKeyFormat(jsonKeys=_jsonKeys))
		newDatas = []
		if res.data:
			for obj in res.data:
				obj = obj.__dict__
				newData = ULTIPA_RESPONSE.Task()
				taskJson = obj.get("taskJson", {})
				newData.param = json.loads(taskJson.get("param", "{}"))
				newData.result = taskJson.get("result")
				task_info = taskJson.get("task_info", {})

				if task_info.get('status_code'):
					task_info["status_code"] = ULTIPA.TaskStatusString[task_info.get("TASK_STATUS")]
				if task_info.get('engine_cost'):
					task_info["engine_cost"] = task_info.get("writing_start_time", 0) - task_info.get("start_time", 0)

				newData.task_info = convertToAnyObject(task_info)
				return_type_get = int(task_info.get('return_type', 0))
				return_type = ULTIPA_RESPONSE.Return_Type()
				return_type.is_realtime = True if return_type_get & ALGO_RETURN_TYPE.ALGO_RETURN_REALTIME else False
				return_type.is_visualization = True if return_type_get & ALGO_RETURN_TYPE.ALGO_RETURN_VISUALIZATION else False
				return_type.is_wirte_back = True if return_type_get & ALGO_RETURN_TYPE.ALGO_RETURN_WRITE_BACK else False
				newData.task_info.__setattr__('return_type', return_type)
				newDatas.append(newData)
			res.data = newDatas
		return res

	def clearTask(self, request: ULTIPA_REQUEST.ClearTask,
				  requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ResponseCommon:
		'''
		:EXP: conn.clearTask(ULTIPA_REQUEST.ClearTask(id=['1'],name='out_degree'))
		:param request: ULTIPA_REQUEST
		:return:
		'''
		uqlMaker = UQLMAKER(command=CommandList.clearTask, commonParams=requestConfig)
		commonP = []
		if request.all:
			commonP.append('*')
		else:

			if request.name and request.status:
				commonP.append(request.name)
				commonP.append(request.status)
			if request.name and not request.status:
				commonP.append(request.name)
				commonP.append('*')
			if not request.name and request.status:
				commonP.append('*')
				commonP.append(request.status)
			if request.id:
				commonP = request.id

		uqlMaker.setCommandParams(commandP=commonP)
		res = self.UqlUpdateSimple(uqlMaker)

		return res

	def stopTask(self, request: ULTIPA_REQUEST.StopTask,
				 requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ResponseCommon:
		uqlMaker = UQLMAKER(command=CommandList.stopTask, commonParams=requestConfig)
		commonP = []
		if request.all:
			commonP = '*'
		if request.id:
			commonP = request.id
		uqlMaker.setCommandParams(commandP=commonP)
		return self.UqlUpdateSimple(uqlMaker)

	def clusterInfo(self,
					requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ClusterInfo:
		self.refreshRaftLeader(redirectHost='', requestConfig=requestConfig)
		result = []
		if not requestConfig.graphName:
			graphSetName = 'default'
		else:
			graphSetName = requestConfig.graphName
		for peer in self.hostManagerControl.getAllHostStatusInfo(graphSetName):
			info = ULTIPA_RESPONSE.Cluster()
			info.status = peer.status
			info.host = peer.host
			info.isLeader = peer.isLeader
			info.isFollowerReadable = peer.isFollowerReadable
			info.isAlgoExecutable = peer.isAlgoExecutable
			info.isUnset = peer.isUnset
			info.cpuUsage = None
			info.memUsage = None
			if peer.status:
				ret = self.stats(requestConfig=ULTIPA_REQUEST.RequestConfig(host=peer.host))
				if ret.status.code == ULTIPA.Code.SUCCESS:
					info.cpuUsage = ret.data.cpuUsage
					info.memUsage = ret.data.memUsage

			result.append(info)
		res = ULTIPA_RESPONSE.Response()
		res.data = result
		return res



from ultipa.operate.base_extra import BaseExtra

from ultipa.types import ULTIPA_REQUEST, ULTIPA, ULTIPA_RESPONSE
from ultipa.utils import UQLMAKER, CommandList


class EdgeExtra(BaseExtra):

	def searchEdge(self, request: ULTIPA_REQUEST.SearchEdge,
				   requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ResponseSearchEdge:
		'''
		:EXP: conn.searchEdge(ULTIPA_REQUEST.SearchMate(id='1',select=['name']))
		:param request: ULTIPA_REQUEST
		:param requestConfig:
		:return:
		'''
		uqlMaker = UQLMAKER(command=CommandList.edges, commonParams=requestConfig)
		if request.id:
			uqlMaker.setCommandParams(request.id)
		elif request.filter:
			uqlMaker.setCommandParams(request.filter)

		uqlMaker.addParam('as', request.select.aliasName)
		uqlMaker.addParam("return", request.select)

		res = self.uqlSingle(uqlMaker)
		if res.status.code != ULTIPA.Code.SUCCESS:
			return res
		return res

	def insertEdge(self, request: ULTIPA_REQUEST.InsertEdge,
				   requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ResponseInsert:
		'''
		:param requestConfig:
		:EXP: conn.insertEdge(ULTIPA_REQUEST.InsertEdge(edges=[{'_from_id':1,'_to_id':2,'name':'TestEdge'}]))
		:param request: ULTIPA_REQUEST
		:return: status
		'''
		uqlMaker = UQLMAKER(command=CommandList.insert, commonParams=requestConfig)
		if request.upsert:
			uqlMaker = UQLMAKER(command=CommandList.upsert, commonParams=requestConfig)
		if request.overwrite:
			uqlMaker.addParam('overwrite', "", required=False)
		if request.schemaName:
			uqlMaker.addParam('into', request.schemaName, required=False)
		uqlMaker.addParam('edges', request.edges)

		if request.isReturnID:
			uqlMaker.addParam('as', "edges")
			uqlMaker.addParam('return', "edges._uuid")
		res = self.uqlSingle(uqlMaker)
		if res.status.code != ULTIPA.Code.SUCCESS:
			return res
		if request.isReturnID:
			if len(res.aliases) > 0:
				res.data = res.items.get(res.aliases[0].alias).data
		return res

	def updateEdge(self, request: ULTIPA_REQUEST.UpdateEdge,
				   requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		:EXP: conn.updateEdge(ULTIPA_REQUEST.UpdateMate(id='1',params={'rank':30}))
		:param request: ULTIPA_REQUEST
		:param requestConfig:
		:return: status
		'''
		uqlMaker = UQLMAKER(command=CommandList.updateEdges, commonParams=requestConfig)
		if request.id:
			uqlMaker.setCommandParams(request.id)
		elif request.filter:
			uqlMaker.setCommandParams(request.filter)
		uqlMaker.addParam("set", request.values)
		uqlMaker.addParam("silent", request.silent)
		res = self.UqlUpdateSimple(uqlMaker)
		return res

	def deleteEdge(self, request: ULTIPA_REQUEST.DeleteEdge,
				   requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		:param requestConfig:
		:EXP: conn.deleteEdge(ULTIPA_REQUEST.DeleteMate(filter={'name':'TestEdge'}))
		:param request: ULTIPA_REQUEST
		:return: status
		'''
		uqlMaker = UQLMAKER(command=CommandList.deleteEdges, commonParams=requestConfig)
		if request.id:
			uqlMaker.setCommandParams(request.id)
		elif request.filter:
			uqlMaker.setCommandParams(request.filter)
		res = self.UqlUpdateSimple(uqlMaker)
		return res



# -*- coding: utf-8 -*-
# @Time    : 2023/7/19 12:06
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : treeNode.py
from ultipa.types.types import ExplainPlan


class TreeNode:

	def __init__(self, explain: ExplainPlan = None):
		self.explain = explain
		self.childNodes = []



# -*- coding: utf-8 -*-
# @Time    : 2023/7/18 09:20
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : hostManagerControl.py
from typing import List
from ultipa.types import ULTIPA
from ultipa.connection.hostManager import HostManager

RAFT_GLOBAL = "global"

class HostManagerControl:
	initHost: str = None
	username: str = None
	password: str = None
	crt: str = None
	allHostManager: dict = {}
	consistency: bool = False

	def __init__(self, initHost: str, username: str, password: str, maxRecvSize: int = -1, crt: str = None,
				 consistency: bool = False):
		self.initHost = initHost
		self.username = username
		self.password = password
		self.maxRecvSize = maxRecvSize
		self.consistency = consistency
		self.crt = crt
		self.allHostManager = {}

	def chooseClientInfo(self, type: int, uql: str, graphSetName: str, useHost: str = None, useMaster: bool = False):
		hostManager = self.getHostManger(graphSetName)
		return hostManager.chooseClientInfo(type, uql, consistency=self.consistency,
											useHost=useHost, useMaster=useMaster)

	def upsetHostManger(self, graphSetName: str, initHost: str):
		hostManager = HostManager(graphSetName=graphSetName, host=initHost, username=self.username,
								  password=self.password, crt=self.crt, maxRecvSize=self.maxRecvSize)
		self.allHostManager[graphSetName] = hostManager
		return hostManager

	def getHostManger(self, graphSetName: str):
		hostManager = self.allHostManager.get(graphSetName)
		if not hostManager:
			hostManager = self.upsetHostManger(graphSetName=graphSetName, initHost=self.initHost)
		return hostManager

	def getAllHosts(self):
		hostManager = self.getHostManger(RAFT_GLOBAL)
		return hostManager.getAllHosts()

	def getAllClientInfos(self, graph):
		return self.getHostManger(graph).getAllClientInfos(ignoreAlgo=True)

	def getAllHostStatusInfo(self, graph):
		all: List[ULTIPA.RaftPeerInfo] = []
		all.extend(self.getHostManger(graph).followersPeerInfos)
		all.append(self.getHostManger(graph).leaderInfos)
		return all


from ultipa.operate.base_extra import BaseExtra
from ultipa.types import ULTIPA_REQUEST, ULTIPA_RESPONSE


class ExportExtra(BaseExtra):

	def export(self, request: ULTIPA_REQUEST.Export,
			   requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ResponseExport:
		res = self.exportData(request, requestConfig)
		return res



# -*- coding: utf-8 -*-
# @Time    : 2023/7/18 11:53
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : clientType.py
class ClientType:
	Default = 0  # 默认
	Algo = 1  # 算法
	Update = 2  # 更新
	Leader = 3  # 主节点



from typing import List, Dict

from ultipa.printer.prettyPrint import PrettyPrint, PrettyTable
from ultipa.types.types import BaseModel, Node, Edge, DataItem, Status, \
	UltipaStatistics, Schema, ReturnReq, BaseUqlReply, ResultType, ResultAlias, ExplainPlan
from ultipa.utils.convert import Any


class Property(BaseModel):
	lte: bool
	index: bool
	propertyName: str
	propertyType: str
	description: str
	schema: str


class PropertyTable(BaseModel):
	name: str
	data: List[Property]

	def __init__(self, name, data):
		self.name = name
		self.data = data


class SchemaTable(BaseModel):
	name: str
	data: List[Schema]

	def __init__(self, name, data):
		self.name = name
		self.data = data


class Response(BaseModel):
	def __init__(self, status: Status = None, data: BaseUqlReply = None,
				 req: ReturnReq = None, statistics: UltipaStatistics = None, aliases: List[ResultAlias] = None):
		self.status = status
		self.data = data
		self.statistics = statistics
		self.req = req
		self.aliases = aliases

	def Print(self):
		pretty = PrettyPrint()
		pretty.prettyStatus(self.status)
		if self.status.code != 0:
			return
		if self.statistics:
			pretty.prettyStatistics(self.statistics)

		if isinstance(self.data, list):
			dict_list = [i.__dict__ for i in self.data]
			pretty.prettyDataList(dict_list)

		if isinstance(self.data, Any):
			dict_list = self.data.__dict__
			pretty.prettyData(dict_list)


class UltipaResponse(BaseModel):
	def __init__(self, status: Status = None, items: Dict = None,
				 req: ReturnReq = None, statistics: UltipaStatistics = None, aliases: List[ResultAlias] = None,
				 explainPlan: List[ExplainPlan] = None):
		self.status = status
		self.items = items
		self.aliases = aliases
		self.req = req
		self.statistics = statistics
		self.explainPlan = explainPlan

	def alias(self, alias: str) -> DataItem:
		if self.items == None:
			return DataItem(alias, None, ResultType.getTypeStr(ResultType.RESULT_TYPE_UNSET))
		if self.items.get(alias):
			return self.items.get(alias)
		return DataItem(alias, None, ResultType.getTypeStr(ResultType.RESULT_TYPE_UNSET))

	def get(self, index: int) -> DataItem:
		if len(self.aliases) - 1 >= index:
			data = self.items.get(self.aliases[index].alias)
			if data:
				return data
			if self.aliases[index].result_type == ResultType.getTypeStr(ResultType.RESULT_TYPE_UNSET):
				return DataItem(None, None, ResultType.getTypeStr(ResultType.RESULT_TYPE_UNSET))
			else:
				return data
		return DataItem(None, None, ResultType.getTypeStr(ResultType.RESULT_TYPE_UNSET))

	def getExplainPlan(self):
		return self.explainPlan

	def Print(self):
		pretty = PrettyPrint()
		pretty.prettyStatus(self.status)
		if self.status.code != 0:
			return
		pretty.prettyStatistics(self.statistics)
		explains = []
		for key in self.items:
			dataItem = self.items.get(key)
			if dataItem.type == ResultType.getTypeStr(ResultType.RESULT_TYPE_NODE):
				pretty.prettyNode(dataItem)

			elif dataItem.type == ResultType.getTypeStr(ResultType.RESULT_TYPE_EDGE):
				pretty.prettyEdge(dataItem)

			elif dataItem.type == ResultType.getTypeStr(ResultType.RESULT_TYPE_TABLE):
				pretty.prettyTable(dataItem)

			elif dataItem.type == ResultType.getTypeStr(ResultType.RESULT_TYPE_PATH):
				pretty.prettyPath(dataItem)

			elif dataItem.type == ResultType.getTypeStr(ResultType.RESULT_TYPE_ATTR):
				pretty.prettyAttr(dataItem)


		if self.explainPlan:
			for explan in self.explainPlan:
				explains.append(explan)
			if explains:
				pretty.prettyTree(explains)


class SchemaResponse(BaseModel):
	def __init__(self, status: Status = None, total_time: int = 0, engine_time: int = 0, data: List[SchemaTable] = None,
				 req=None, statistics: UltipaStatistics = None):
		self.status = status
		self.data = data
		self.total_time = total_time
		self.engine_time = engine_time
		self.req = req
		self.statistics = statistics

	def Print(self):
		pretty = PrettyPrint()
		pretty.prettyStatus(self.status)
		if self.status.code != 0:
			return
		if self.statistics:
			pretty.prettyStatistics(self.statistics)
		for data in self.data:
			propertList = []
			if len(data.data) > 0:
				for index, header in enumerate(data.asTable().getHeaders()):
					propertList.append(header.get("property_name"))
			else:
				return

			y = PrettyTable(propertList, title=f"Alias: {data.name}")
			for row in data.asTable().getRows():
				y.add_row(row)
			print(y)



class ResponseCommon(Response):
	data: None



class User(BaseModel):
	username: str
	create_time: str
	last_login_time: str
	graphPrivileges: dict
	systemPrivileges: List[str]
	policies: List[str]


class Index(BaseModel):
	name: str
	properties: str
	schema: str
	status: str


class IndexTable:
	name: str
	data: List[Index]


class Stat(BaseModel):
	cpuUsage: str
	memUsage: str
	company: str
	cpuCores: str
	expiredDate: str
	serverType: str
	version: str


class Graph(BaseModel):
	id: str
	name: str
	totalNodes: str
	totalEdges: str
	description: str
	status: str


class Privilege(BaseModel):
	graphPrivileges: List[str]
	systemPrivileges: List[str]


class Policy(BaseModel):
	name: str
	policies: List[str]
	graphPrivileges: dict
	systemPrivileges: List[str]


class Top(BaseModel):
	process_id: str
	process_uql: str
	duration: str


class Return_Type(BaseModel):
	is_realtime: bool
	is_visualization: bool
	is_wirte_back: bool


class Task_info(BaseModel):
	task_id: int
	algo_name: str
	start_time: int
	writing_start_time: int
	end_time: int
	time_cost: int
	TASK_STATUS: int
	status_code: str
	engine_cost: int
	return_type: Return_Type


class Task(BaseModel):
	param: dict
	task_info: Task_info
	result: dict


class SearchKhop(BaseModel):
	values: dict
	nodes: List[Node]


class Path(BaseModel):
	nodes: List[Node]
	edges: List[Edge]


class SearchPath(BaseModel):
	paths: List[Path]


class NodeSpread(SearchPath):
	pass


class AutoNet(SearchPath):
	pass


class AlgoResultOpt(BaseModel):
	can_realtime: bool
	can_visualization: bool
	can_write_back: bool


class Algo(BaseModel):
	param: dict
	name: str
	result_opt: AlgoResultOpt


class SearchAB(Response):
	data: SearchPath


class ResponseAutoNet(Response):
	data: AutoNet


class ResponseGraph(Response):
	data: Graph


class ResponseListGraph(Response):
	data: List[Graph]


class ResponseListIndex(UltipaResponse):
	data: List[IndexTable]


class ResponseListFulltextIndex(ResponseListIndex):
	pass


class ResponseSearchEdge(UltipaResponse):
	items: List[DataItem]


class ResponseSearchNode(UltipaResponse):
	items: List[DataItem]


class ResponseBulk:
	uuids: List[int]
	errorItem: Dict


class ResponseInsert(Response):
	data: List[int]


class ResponseBulkInsert(Response):
	data: ResponseBulk


class ResponseBatchAutoInsert(Response):
	data: Dict[str, ResponseBulk]


class InsertResponse(Response):
	data: ResponseBulk


class ResponseNodeSpread(Response):
	data: NodeSpread


class ResponseListPolicy(Response):
	data: List[Policy]


class ResponsePolicy(Response):
	data: Policy


class ResponseListPrivilege(Response):
	data: List[Privilege]


class ResponseListProperty(Response):
	data: List[PropertyTable]


class ResponseGetProperty(Response):
	data: List[Property]


class ResponseSearchAB(Response):
	data: SearchAB


class ResponseSearchKhop(Response):
	data: SearchKhop


class ResponseListTop(Response):
	data: List[Top]


class ResponseListTask(Response):
	data: List[Task]


class ResponseUser(Response):
	data: User


class ResponseListUser(Response):
	data: List[User]


class ResponseListAlgo(Response):
	data: List[Algo]


class Cluster:
	host: str
	status: bool
	cpuUsage: str
	memUsage: str
	isLeader: bool
	isFollowerReadable: bool
	isAlgoExecutable: bool
	isUnset: bool


class ClusterInfo(Response):
	data: List[Cluster]


class ResponseStat(Response):
	data: Stat


class ResponseExport(Response):
	data: List[Node]





from ultipa.operate.base_extra import BaseExtra
from ultipa.types import ULTIPA_REQUEST, ULTIPA, ULTIPA_RESPONSE
from ultipa.utils import UQLMAKER, CommandList


class IndexExtra(BaseExtra):

	def showIndex(self, request: ULTIPA_REQUEST.ShowIndex = None,
				  requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ResponseListIndex:
		'''
		:param requestConfig:
		:EXP: conn.showIndex(ULTIPA_REQUEST.ShowIndex())
		:param request: ULTIPA_REQUEST
		:return:
		'''
		if request:
			command = request.DBtype == ULTIPA.DBType.DBNODE and CommandList.showNodeIndex or CommandList.showEdgeIndex
		else:
			command = CommandList.showIndex
		uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
		res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
		return res

	def showFulltext(self, request: ULTIPA_REQUEST.ShowFulltext = None,
					 requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.ResponseListIndex:
		'''
		:param requestConfig:
		:EXP: conn.showIndex(ULTIPA_REQUEST.ShowIndex())
		:param request: ULTIPA_REQUEST
		:return:
		'''
		if request:
			command = request.DBtype == ULTIPA.DBType.DBNODE and CommandList.showNodeFulltext or CommandList.showEdgeFulltext
		else:
			command = CommandList.showFulltext
		uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
		res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
		return res

	def createIndex(self, request: ULTIPA_REQUEST.CreateIndex,
					requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		:param requestConfig:
		:EXP: conn.createIndex(ULTIPA_REQUEST.CreatIndex(node_property='name'))
		:param request: ULTIPA_REQUEST
		:return:
		'''
		command = request.DBtype == ULTIPA.DBType.DBNODE and CommandList.createNodeIndex or CommandList.createEdgeIndex
		commandP = request.toString
		uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
		res = self.uqlSingle(uqlMaker=uqlMaker)
		return res

	def createFulltext(self, request: ULTIPA_REQUEST.CreateFulltext,
					   rquestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		command = request.DBtype == ULTIPA.DBType.DBNODE and CommandList.createNodeFulltext or CommandList.createEdgeFulltext
		commandP = [request.toString, request.name]
		uqlMaker = UQLMAKER(command=command, commonParams=rquestConfig)
		uqlMaker.setCommandParams(commandP=commandP)
		res = self.uqlSingle(uqlMaker=uqlMaker)
		return res

	def dropIndex(self, request: ULTIPA_REQUEST.DropIndex,
				  requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		:EXP: conn.dropIndex(ULTIPA_REQUEST.DropIndex(node_property='node'))
		:param request: ULTIPA_REQUEST
		:return:
		'''
		command = request.DBtype == ULTIPA.DBType.DBNODE and CommandList.dropNodeIndex or CommandList.dropEdgeIndex
		commandP = request.toString

		uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
		res = self.uqlSingle(uqlMaker=uqlMaker)
		return res

	def dropFulltext(self, request: ULTIPA_REQUEST.DropFulltext,
					 requestConfig: ULTIPA_REQUEST.RequestConfig = ULTIPA_REQUEST.RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		command = request.DBtype == ULTIPA.DBType.DBNODE and CommandList.dropNodeFulltext or CommandList.dropEdgeFulltext
		uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
		uqlMaker.setCommandParams(request.fulltextName)
		res = self.uqlSingle(uqlMaker=uqlMaker)
		return res



# -*- coding: utf-8 -*-
# @Time    : 2022/7/1 7:42 下午
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : ResposeFormat.py

import json
from typing import Iterable, List, Dict

class ResponseKeyFormat:
    def __init__(self, jsonKeys: List[str] = None, boolKeys: List[str] = None, keyReplace: object = None,
                 dataFormat:  List[str] = None, convert2Int: List[str] = None):
        self.jsonKeys = jsonKeys
        self.boolKeys = boolKeys
        self.keyReplace = keyReplace
        self.dataFormat = dataFormat
        self.convert2Int = convert2Int

    def changeKeyValue(self,objs):
        if objs:
            objs_list = []
            if type(objs) != list:
                objs_list = [objs]
            else:
                objs_list = objs
            for obj in objs_list:
                if self.jsonKeys:
                    for key in self.jsonKeys:
                        if key in obj:
                            obj[key] = json.loads(obj[key])
                if self.boolKeys:
                    for key in self.boolKeys:
                        if key in obj:
                            obj[key] = obj[key] == "true"
                if self.keyReplace:
                    for key in self.keyReplace.keys():
                        if key in obj:
                            obj[self.keyReplace[key]] = obj[key]
                            del obj[key]

                if self.convert2Int:
                    for key in self.convert2Int:
                        if key in obj:
                            obj[key] = int(obj[key])

                if self.dataFormat:
                    for gp in obj:
                        if gp in self.dataFormat:
                            dlist = []
                            for d in obj.get(gp):
                                dlist.append({'name': list(d.keys())[0], 'values': list(d.values())[0]})
                            obj[gp] = dlist
        return objs


from enum import Enum
from typing import List

from ultipa.utils.errors import ParameterException
from ultipa.utils.ufilter.ufilter import FilterBase


class FilterEnum(Enum):
    EQ = "=="
    NEQ = "!="
    LT = "<"
    LTE = "<="
    GT = ">"
    GTE = ">="
    BT = "<>"
    BTE = "<=>"
    OR = "||"
    AND = "&&"
    NOT = "!"
    IN = "in"
    NIN = "nin"


class Filter(FilterBase):
    # def __init__(self, name: str, value: object = None):
    #     self.propertyName = name
    #     self.value = value

    def builder(self):
        ...


class UltipaFilter(Filter):
    def __init__(self, schema: str = 'default', filterType: FilterEnum = None, property: str = None,
                 value: any = None):
        self.propertyName = property
        self.value = value
        self.opt = filterType
        self.schemaName = '@' + schema

    def builder(self):
        # if not self.propertyName and not self.value:
        #     return "{%s}"%self.schemaName.name
        # if not self.schemaName:
        #     return self.value
        if self.value == 0:
            self.value = '0'
        if self.propertyName and (not self.opt or self.value == None):
            raise ParameterException()

        if self.schemaName and self.propertyName and self.value != None and self.opt:
            if self.propertyName in ['_uuid', '_from_uuid', '_to_uuid']:
                return "%s %s %s" % (self.propertyName, self.opt.value, self.value)
            if self.propertyName in ['_id', '_from', '_to']:
                return "%s %s '%s'" % (self.propertyName, self.opt.value, self.value)
            if isinstance(self.value, list) or isinstance(self.value, int):
                return "%s.%s %s %s" % (self.schemaName, self.propertyName, self.opt.value, self.value)
            return "%s.%s %s '%s'" % (self.schemaName, self.propertyName, self.opt.value, self.value)
        if self.propertyName and self.value != None and self.opt:
            if isinstance(self.value, list) or isinstance(self.value, int):
                return "%s %s %s" % (self.propertyName, self.opt.value, self.value)
            return "%s %s '%s'" % (self.propertyName, self.opt.value, self.value)
        if self.schemaName and not self.propertyName:
            return "%s" % self.schemaName

class UltipaAndFilter(Filter):
    def __init__(self,values:List[UltipaFilter]):
        # super().__init__(propertyName, value)
        self.values=values

    def builder(self):
        va = []
        for i in self.values:
            if isinstance(i, Filter):
                va.append(i.builder())
            else:
                va.append(i)
        return " && ".join(va)


class UltipaOrFilter(Filter):
    def __init__(self,values:List[UltipaFilter]):
        # super().__init__(propertyName, value)
        self.values=values

    def builder(self):
        va = []
        for i in self.values:
            if isinstance(i, Filter):
                va.append(i.builder())
            else:
                va.append(i)
        return " || ".join(va)



class UltipaFilterList(FilterBase):

    def __init__(self, *args: UltipaFilter):
        self.firstFilter = args
        print(self.firstFilter)

    # def builder(self):
    #     for i in


# class LtFilter(Filter):
#     def __init__(self,schemaName:Schema, propertyName: str=None, value: any=None):
#         super().__init__(propertyName,value)
#         self.opt = FilterEnum.LT
#         self.schemaName = schemaName.name
#
#     def builder(self):
#         if not self.propertyName and not self.value:
#             return "{%s}"%self.schemaName
#         if not self.schemaName:
#             return self.value


if __name__ == '__main__':
    # ret = UltipaFilter(Schema('test'),filterType=FilterEnum.EQ)
    ret = UltipaFilter(filterType=FilterEnum.BTE, propertyName='test', value=1, schemaName='test')
    # ret1 = ret.builder()
    # print(ret1)
    # UltipaFilterList(ret, ret, ret)

    ret= UltipaAndFilter([ret,ret])
    print(ret.builder())




import json
from typing import List



class BaseModel:
    def toJSON(self, pretty=False):
        try:
            if pretty:
                return json.dumps(self, default=lambda o: o.__dict__,
                                  sort_keys=True, indent=4, ensure_ascii=False)
            else:

                return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, ensure_ascii=False)

        except Exception as e:
            return self

    def toDict(self):
        return json.loads(self.toJSON())

    def __str__(self):
        return str(self.__dict__)



class Any(BaseModel):
    def __str__(self):
        return  str(self.__dict__)
    pass



def convertToAnyObject(dict1: dict):
    obj = Any()
    for k in dict1.keys():
        v = dict1[k]
        if isinstance(v, list):
            for i, n in enumerate(v):
                if isinstance(n, dict):
                    v[i] = convertToAnyObject(n)
        # if isinstance(v, dict):
        #     v = convertToAnyObject(v)
        obj.__setattr__(k, v)
    return obj


def convertToListAnyObject(list1: List[dict]):
    if not list1 and isinstance(list1,list):
        return list1
    if not list1:
        return
    newList = []
    for dict1 in list1:
        newList.append(convertToAnyObject(dict1))
    return newList


def convertAlgoList2AnyObject(list1: List[dict]):
    if not list1 and isinstance(list1,list):
        return list1
    if not list1:
        return
    newList = []
    for dict1 in list1:
        if not dict1.get("write_to_stats_parameters"):
            dict1.update({"write_to_stats_parameters":None})
        if not dict1.get("write_to_db_parameters"):
            dict1.update({"write_to_db_parameters":None})
        if not dict1.get("write_to_file_parameters"):
            dict1.update({"write_to_file_parameters":None})
        newList.append(convertToAnyObject(dict1))
    return newList

def convertTableToListAnyObject(list1: List[dict],headers):
    if not list1 and isinstance(list1,list):
        return list1
    if not list1:
        return list1
    if not headers:
        return list1

    newList = []
    for data in list1:
        dic = {}
        for index,header in enumerate(headers):
            dic.update({header.get("property_name"):data[index]})
        newList.append(convertToAnyObject(dic))

    return newList

def convertTableToDict(table_rows,headers):
    newList = []
    for data in table_rows:
        dic = {}
        for index,header in enumerate(headers):
            dic.update({header.get("property_name"):data[index]})
        newList.append(dic)
    return newList
